c two_body_velocity
c
c a,b is projectile
c c,d is fragment
c
c input theta_lab, ma, ka, mb, kb, mc, md
c output return value, kc, kd
c
c /////////////////////////////////////////////////////////////////////>
      real function f_two_body_beta
     &     (theta_lab, theta_cm, ma, ka, mb, kb, mc, kc, md, kd)
      implicit none

c using functions ---------------------------------------->
      real f_energy, f_momentum, f_beta, f_gamma, f_norm
      real f_theta_lab_to_cm
      real PI
c --------------------------------------------------<

c variables ---------------------------------------->
      real theta_lab, theta_cm, ma, ka, mb, kb, mc, kc, md, kd

      real ea_lab, pa_lab
      real eb_lab, pb_lab
      real ea_cm, pa_cm
      real eb_cm, pb_cm

      real ec_lab,pc_lab,pc_lab_x,pc_lab_y
      real ed_lab,pd_lab,pd_lab_x,pd_lab_y

      real ec_cm, pc_cm, pc_cm_x,pc_cm_y
      real ed_cm, pd_cm, pd_cm_x,pd_cm_y

      real beta_c_cm, beta_d_cm
c ---------------------------------------------------------<

c lorentz  transformation parameter --------->
      real beta_lo, gamma_lo
c -------------------------------------------<

c cal energy and momentum ------------------------->
      ea_lab = f_energy(ma, ka)
      pa_lab = f_momentum(ma, ka)
      eb_lab = f_energy(mb, kb)
      pb_lab = f_momentum(mb, kb)
c -------------------------------------------------<

c lorenzs boost parameter ----------------------------->
      beta_lo = f_beta(ea_lab + eb_lab, pa_lab + pb_lab)
      gamma_lo = f_gamma(beta_lo)
c -----------------------------------------------------<

c lab -> cm ------------------------------------------>
      call lorentz(beta_lo, ea_lab, pa_lab, ea_cm, pa_cm)
      call lorentz(beta_lo, eb_lab, pb_lab, eb_cm, pb_cm)
c ----------------------------------------------------< 

c a + b -> c + d ------------------------------------>
      call ab_to_cd(ea_cm + eb_cm, ma, mb, mc, md,
     &     ec_cm, pc_cm, ed_cm, pd_cm)

      beta_c_cm = f_beta(ec_cm, pc_cm)
      beta_d_cm = f_beta(ed_cm, pd_cm)
c -------------------------------------------------<

c theta_lab -> theta_cm ---------------------------->
      theta_cm = f_theta_lab_to_cm
     &     (theta_lab, beta_c_cm, gamma_lo, beta_lo)
c --------------------------------------------------<

c      write(*,*) beta_lo, gamma_lo, theta_lab, theta_cm

c ec_cm, ed_cm, pc_cm,pd_cm -> ec_lab, ed_lab, pc_lab,pd_lab -------->
      pc_cm_x = pc_cm * cos(theta_cm)
      pc_cm_y = pc_cm * sin(theta_cm)
      pd_cm_x = pd_cm * cos(theta_cm + PI())
      pd_cm_y = pd_cm * sin(theta_cm + PI())
                
      call lorentz(-beta_lo, ec_cm, pc_cm_x, ec_lab, pc_lab_x)
      call lorentz(-beta_lo, ed_cm, pd_cm_x, ed_lab, pd_lab_x)

      pc_lab_y = pc_cm_y
      pd_lab_y = pd_cm_y

      pc_lab = f_norm(pc_lab_x, pc_lab_y, 0.)
      pd_lab = f_norm(pd_lab_x, pd_lab_y, 0.)

      kc = ec_lab - mc
      kd = ed_lab - md
c -------------------------------------------------------------------<

c should be same
c      write(*,*) tan(theta_lab), pc_lab_y/pc_lab_x

c should be mass
c      write(*,*) "moto      ", ma, mb, mc, md
c      write(*,*) "modoruhazu",
c     &     f_mass(ea_lab, pa_lab), f_mass(eb_lab, pb_lab),
c     &     f_mass(ec_lab, pc_lab), f_mass(ed_lab, pd_lab) 

c return data ------------------------------------------------------->
      f_two_body_beta = f_beta(ec_lab, pc_lab)
c -------------------------------------------------------------------<

c      write(*,*) "v ", two_body_beta*C_LIGHT(), "  beta ", two_body_beta 

      return 
      end
c /////////////////////////////////////////////////////////////////////<

c theta lab to theta cm
c /////////////////////////////////////////////////////////////////////>
      real function f_theta_lab_to_cm
     &     (theta_lab, beta_cm, gamma_lo, beta_lo)
      implicit none

      real theta_lab, beta_cm, gamma_lo, beta_lo
      real p, q, a, b, c

      p = gamma_lo * tan(theta_lab)
      q = beta_lo/beta_cm
      
      a = p**2 + 1.0
      b = 2.0 * p**2 * q
      c = p**2 * q**2 - 1.0

      f_theta_lab_to_cm = acos((-b + sqrt(b**2 - 4.0*a*c))/(2.0*a))

      return
      end
c /////////////////////////////////////////////////////////////////////<

c a + b -> c + d
c /////////////////////////////////////////////////////////////////////>
      subroutine ab_to_cd(e_all_cm, ma, mb, mc, md,
     &     ec_cm, pc_cm, ed_cm, pd_cm)
      implicit none

      real e_all_cm, ma, mb, mc, md, ec_cm, pc_cm, ed_cm, pd_cm

      pc_cm = sqrt((mc**2 - md**2)**2 - 2.*e_all_cm**2*(mc**2 + md**2)
     &     +e_all_cm**4)/(2.*e_all_cm)
      ec_cm = sqrt(mc**2 + pc_cm**2)

      pd_cm = pc_cm
      ed_cm = sqrt(md**2 + pd_cm**2)

      return
      end
c /////////////////////////////////////////////////////////////////////<

